#include "Wire.h"          // I2C library
#include "time.h"          // C Time library

#define RTC_ADDRESS 0x68   // I2C address
struct tm timeinfo;

void setup() {
  Serial.begin(115200);
  Wire.begin();
  //setTime(&timeinfo);    // Shuld be set at beginning
}

void loop() {
  getTime(&timeinfo);      // Readout time/date
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
  delay(1000);             // Wait for 1 sec
}

//--- Convert decimal numbers to BCD -----------------------
byte decToBcd(byte val) {
  return ( (val / 10 * 16) + (val % 10) );
}
//--- Convert BCD numbers to decimal ----------------------
byte bcdToDec(byte val) {
  return ( (val / 16 * 10) + (val % 16) );
}

//--- Set RTC time/date ------------------------------------
void setTime(struct tm* time) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0); // set register pointer to 00h
  Wire.write(decToBcd(time->tm_sec));   // set seconds
  Wire.write(decToBcd(time->tm_min));   // set minutes
  Wire.write(decToBcd(time->tm_hour));  // set hours
  Wire.write(time->tm_wday + 1);        // set day of week (1=Sunday, 7=Saturday)
  Wire.write(decToBcd(time->tm_mday));  // set date (1 to 31)
  Wire.write(decToBcd(time->tm_mon) + 1); // set month
  Wire.write(decToBcd(time->tm_year - 100)); // year from 2000 (0 to 99)
  Wire.endTransmission();
}

//--- Read time/date from RTC ------------------------------
void getTime(struct tm* time) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0);                     // set register pointer to 00h
  Wire.endTransmission(false);
  Wire.requestFrom(RTC_ADDRESS, 7);  // request 7 bytes starting from register 00h
  time->tm_sec = bcdToDec(Wire.read() & 0x7f);
  time->tm_min = bcdToDec(Wire.read());
  time->tm_hour = bcdToDec(Wire.read() & 0x3f);
  time->tm_wday = bcdToDec(Wire.read() - 1);
  time->tm_mday = bcdToDec(Wire.read());
  time->tm_mon = bcdToDec(Wire.read() - 1);
  time->tm_year = bcdToDec(Wire.read()) + 100;
}
