#include "Wire.h"          // https://docs.espressif.com/projects/arduino-esp32/en/latest/api/i2c.html
#include "oled.h"          // https://github.com/durydevelop/arduino-lib-oled
#include <WiFi.h>          // https://docs.espressif.com/projects/arduino-esp32/en/latest/api/wifi.html
#include "time.h"          // https://www.cplusplus.com/reference/ctime/
#include "secrets.h"       // WiFi SSID & password

#define RTC_ADDRESS 0x68   // I2C eszköz címe
struct tm timeinfo;
static char msg[20];      // character buffer

OLED display = OLED(18,19,NO_RESET_PIN, 0x3C, 128,32, false); // SSD1306
//OLED display = OLED(18,19,NO_RESET_PIN,0x3C,128,64, false); // SSD1306
//OLED display = OLED(18, 19, NO_RESET_PIN, 0x3C, 128, 64, true); // SH1106

void setup() {
  Serial.begin(115200);
  Wire.begin();
  display.begin();
  display.clear();
  display.draw_string(4, 8, "RTC clock", OLED::DOUBLE_SIZE);
  display.display();
  setup_wifi();            // Connecting to WiFi AP
  setup_sntp();            // Requesting time from NTP server
  WiFi.disconnect(true);   // disconnect WiFi
  WiFi.mode(WIFI_OFF);     // as it's no longer needed
}

void loop() {
  getTime(&timeinfo);      // display time/date
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
  display.clear();
  strftime (msg, 15, "%Y-%b-%d %a ", &timeinfo);
  display.draw_string(16, 1, msg);           // Display date
  strftime (msg, 10, "%T ", &timeinfo);      // Display time
  display.draw_string(8, 12, msg, OLED::DOUBLE_SIZE);
  display.display();                         // Refresh screen
  delay(1000);             // every second
}

//--- Connecting to a local WiFi AP ------------------------
void setup_wifi() {
  WiFi.mode(WIFI_STA);
  Serial.print("\r\nConnecting to ");
  Serial.println(WIFI_SSID);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
}

//--- Requesting local time from NTP server ----------------
void setup_sntp() {
  const char* ntpServer = "hu.pool.ntp.org";
  // TimeZone rule for Europe/Budapest including daylight adjustment rules (optional)
  // See at: https://leo.leung.xyz/wiki/Timezone
  const char* time_zone = "CET-1CEST,M3.5.0,M10.5.0/3";
  configTzTime(time_zone, ntpServer);
  Serial.println("Connecting to sntp server");
  while (!getLocalTime(&timeinfo)) {
    Serial.print(".");
  }
  setTime(&timeinfo);
  Serial.println(" NTP connected");
  Serial.println(&timeinfo, "NTP time: %A, %B %d %Y %H:%M:%S");
}

//--- Convert decimal numbers to BCD -----------------------
byte decToBcd(byte val) {
  return ( (val / 10 * 16) + (val % 10) );
}
//--- Convert BCD numbers to decimal ----------------------
byte bcdToDec(byte val) {
  return ( (val / 16 * 10) + (val % 16) );
}

//--- Set RTC time/date ------------------------------------
void setTime(struct tm* time) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0); // set register pointer to 00h
  Wire.write(decToBcd(time->tm_sec));   // set seconds
  Wire.write(decToBcd(time->tm_min));   // set minutes
  Wire.write(decToBcd(time->tm_hour));  // set hours
  Wire.write(time->tm_wday + 1);        // set day of week (1=Sunday, 7=Saturday)
  Wire.write(decToBcd(time->tm_mday));  // set date (1 to 31)
  Wire.write(decToBcd(time->tm_mon) + 1); // set month
  Wire.write(decToBcd(time->tm_year - 100)); // year from 2000 (0 to 99)
  Wire.endTransmission();
}

//--- Read time/date from RTC ------------------------------
void getTime(struct tm* time) {
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0);                     // set register pointer to 00h
  Wire.endTransmission(false);
  Wire.requestFrom(RTC_ADDRESS, 7);  // request 7 bytes starting from register 00h
  time->tm_sec = bcdToDec(Wire.read() & 0x7f);
  time->tm_min = bcdToDec(Wire.read());
  time->tm_hour = bcdToDec(Wire.read() & 0x3f);
  time->tm_wday = bcdToDec(Wire.read() - 1);
  time->tm_mday = bcdToDec(Wire.read());
  time->tm_mon = bcdToDec(Wire.read() - 1);
  time->tm_year = bcdToDec(Wire.read()) + 100;
}
